#include <algorithm> // 'std::swap', etc.
#include <cassert>
#include <cstdlib> // 'std::srand', etc.
#include <ctime>

#include "Sort_illustration.h"

void Sort_illustration::sort(int left, int right, int& cost) {
    assert(left < right);
    for (int i = left; i < right; i++) {
        for (int j = i + 1; j <= right; j++) {
            cost++;
            if (_representation[i] > _representation[j]) std::swap(_representation[i], _representation[j]);
        }
    }
}

int Sort_illustration::split(int left, int right, int& cost) {
    int pivot = left;
    int value = _representation[left];
    for (int i = left + 1; i <= right; i++) {
        cost++;
        if (_representation[i] < value) {
            pivot++;
            std::swap(_representation[pivot], _representation[i]);
        }
    }
    std::swap(_representation[left], _representation[pivot]);
    return pivot;
}

void Sort_illustration::quick_sort(int left, int right, int& cost) {
    if (right - left <= 0) return;
    int pivot = split(left, right, cost);
    quick_sort(left, pivot - 1, cost);
    quick_sort(pivot + 1, right, cost);
}

Sort_illustration::Sort_illustration(int size) : _size(size) {
    assert(_size > 0);
    _representation = new int[_size];
    time_t t;
    std::srand(std::time(&t));
    for (int i = 0; i < _size; i++) _representation[i] = std::rand();
}

Sort_illustration::~Sort_illustration() {
    delete[] _representation;
}

int Sort_illustration::sort(Speed s) {
    int cost = 0;
    switch (s) {
        case Slow: sort(0, _size - 1, cost);
            break;
        case Quick: quick_sort(0, _size - 1, cost);
            break;
        default:;
    }
    return cost;
}